#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "convexMRF.h"

ConvexMRF::ConvexTerm::ConvexTerm(int _breakpointNumMax, void (*err_function)(char *))
{
	breakpointNumMax = _breakpointNumMax;
	points = new Point[breakpointNumMax];
	buf = new Point*[2*breakpointNumMax];
	if (!points || !buf) { if (err_function) (*err_function)("Not enough memory!"); exit(1); }

	Reset();
}

ConvexMRF::ConvexTerm::~ConvexTerm()
{
	delete [] points;
	delete [] buf;
}

void ConvexMRF::ConvertConvexTerm(ConvexTermInternal& to, ConvexTerm& from, bool reverse)
{
	int N = from.breakpointNum;
	if (N == 0)
	{
		to.breakpointNum = 0;
		to.breakpoints = NULL;
		to.slopes = (REAL *) Malloc(sizeof(REAL), error_function);
		to.slopes[0] = (reverse) ? -from.slope0 : from.slope0;
		to.flow = 0;
		return;
	}

	// 1. Sort breakpoints in from.points

	// mergesort
	int i, j;
	ConvexTerm::Point** bufSrc = from.buf;
	ConvexTerm::Point** bufDst = from.buf + N;
	ConvexTerm::Point** pSrc1;
	ConvexTerm::Point** pSrc2;
	ConvexTerm::Point** pSrc1End;
	ConvexTerm::Point** pSrc2End;
	ConvexTerm::Point** pDst;
	for (i=0; i<(N&(~1)); i+=2)
	{
		if (from.points[i].breakpoint < from.points[i+1].breakpoint)
		{
			bufSrc[i]   = &from.points[i];
			bufSrc[i+1] = &from.points[i+1];
		}
		else
		{
			bufSrc[i]   = &from.points[i+1];
			bufSrc[i+1] = &from.points[i];
		}
	}
	if (i != N) bufSrc[i]   = &from.points[i];

	int step = 2;
	for (step=2; step<N; step*=2)
	{
		pSrc2End = bufSrc;
		pDst = bufDst;
		while ( 1 )
		{
			pSrc1 = pSrc2End;
			pSrc1End = pSrc1 + step;
			if (pSrc1End >= bufSrc + N)
			{
				memcpy(pDst, pSrc1, (int)((char*)(bufSrc + N) - (char*)pSrc1));
				break;
			}
			pSrc2 = pSrc1End;
			pSrc2End = pSrc2 + step;
			if (pSrc2End > bufSrc + N) pSrc2End = bufSrc + N;
			while ( 1 )
			{
				if ((*pSrc1)->breakpoint < (*pSrc2)->breakpoint)
				{
					*pDst ++ = *pSrc1 ++;
					if (pSrc1 == pSrc1End)
					{
						i = (int)((char*)pSrc2End - (char*)pSrc2);
						memcpy(pDst, pSrc2, i);
						pDst = (ConvexTerm::Point**) ((char*)pDst + i);
						break;
					}
				}
				else
				{
					*pDst ++ = *pSrc2 ++;
					if (pSrc2 == pSrc2End)
					{
						i = (int)((char*)pSrc1End - (char*)pSrc1);
						memcpy(pDst, pSrc1, i);
						pDst = (ConvexTerm::Point**) ((char*)pDst + i);
						break;
					}
				}
			}
		}
		pDst = bufDst;
		bufDst = bufSrc;
		bufSrc = pDst;
	}

	// 2. Count the number of breakpoints (exclude duplicates)
	to.breakpointNum = N;
	for (i=0; i<N-1; i++)
	{
		assert(bufSrc[i]->breakpoint <= bufSrc[i+1]->breakpoint);
		if (bufSrc[i]->breakpoint == bufSrc[i+1]->breakpoint) to.breakpointNum --;
	}

	// 3. Allocate memory
	to.breakpoints = (Label*) Malloc(to.breakpointNum*sizeof(Label) + (to.breakpointNum + 1)*sizeof(REAL), error_function);
	to.slopes = (REAL *) (((char*)to.breakpoints) + to.breakpointNum*sizeof(Label));

	// 4. Copy breakpoints and slopes
	if (!reverse)
	{
		to.slopes[0] = from.slope0;
		for (i=0, j=0; i<N; i++)
		{
			if (j>0 && to.breakpoints[j-1] == bufSrc[i]->breakpoint)
			{
				to.slopes[j] += bufSrc[i]->slope;
			}
			else
			{
				to.breakpoints[j] = bufSrc[i]->breakpoint;
				to.slopes[j+1] = to.slopes[j] + bufSrc[i]->slope;
				j ++;
			}
		}
	}
	else
	{
		to.slopes[to.breakpointNum] = -from.slope0;
		for (i=0, j=0; i<N; i++)
		{
			if (j>0 && to.breakpoints[to.breakpointNum - j] == bufSrc[i]->breakpoint)
			{
				to.slopes[to.breakpointNum - j] += bufSrc[i]->slope;
			}
			else
			{
				to.breakpoints[to.breakpointNum - j - 1] = bufSrc[i]->breakpoint;
				to.slopes[to.breakpointNum - j - 1] = to.slopes[to.breakpointNum - j] + bufSrc[i]->slope;
				j ++;
			}
		}
	}

	// Almost done.
	to.flow = 0;
}

ConvexMRF::ConvexMRF(int _nodeNum, int _edgeNumMax, void (*err_function)(char *))
	: nodeNum(_nodeNum),
	  edgeNum(0),
	  edgeNumMax(_edgeNumMax),
	  error_function(err_function),
	  memoryBlockFirst(NULL),
	  memoryBlockLast(NULL)
{
	nodes = new Node[nodeNum];
	edges = new Edge[edgeNumMax];
	if (!nodes || !edges) { if (err_function) (*err_function)("Not enough memory!"); exit(1); }

	memset(nodes, 0, nodeNum*sizeof(Node));
	memset(edges, 0, edgeNumMax*sizeof(Edge));

	static REAL ZERO_SLOPE = 0;

	Node* i;
	for (i=nodes; i<nodes+nodeNum; i++)
	{
		i->D.slopes = &ZERO_SLOPE;
	}
}

ConvexMRF::~ConvexMRF()
{
	delete [] nodes;
	delete [] edges;
	while (memoryBlockFirst)
	{
		MemoryBlock* next = memoryBlockFirst->next;
		delete [] ((char*) memoryBlockFirst);
		memoryBlockFirst = next;
	}
}

bool ConvexMRF::Up()
{
	GraphType::node_id _i;
	Node* i;
	Node* j;
	Edge* e;
	REAL gradUp, gradDown, flow;
	Label d;
	bool isChanged = false;

	/////////////////////////
	//      MaxflowUp      //
	/////////////////////////
	graph->add_node(nodeNum);
	for (_i=0, i=nodes; i<nodes+nodeNum; _i++, i++)
	{
		gradUp = (i->x == Kmax) ? REALMAX : i->D.GetGradientUp(i->x);
		graph->add_tweights(_i, gradUp, 0);

		for (e=i->firstForward; e; e=e->nextForward)
		{
			j = e->j;
			if (!e->V.isBreakpoint(j->x - i->x, gradUp, gradDown)) continue;

			graph->add_edge(_i, (int)(j - nodes), gradUp, -gradDown);
		}
	}

	graph->maxflow();

	GraphType::arc_id a = graph->get_first_arc();
	for (i=nodes; i<nodes+nodeNum; i++)
	{
		for (e=i->firstForward; e; e=e->nextForward)
		{
			j = e->j;
			if (!e->V.isBreakpoint(j->x - i->x, gradUp, gradDown)) continue;

			flow = graph->get_rcap(a) - gradUp;

			i->D.flow += flow;
			j->D.flow -= flow;
			e->V.flow += flow;

			a = graph->get_next_arc(a);
			a = graph->get_next_arc(a);
		}

		// initialize DijsktraUp
		d = i->D.GetMaxNonincreasing(i->x, Kmax) - i->x;
		i->next = dijkstraArrayFirst[d];
		if (dijkstraArrayFirst[d]) dijkstraArrayFirst[d]->prev = i;
		dijkstraArrayFirst[d] = i;
		i->prev = NULL;
		i->xMax = i->x + d;
	}

	graph->reset();

	/////////////////////////
	//      DijkstraUp     //
	/////////////////////////
	for (d=0; d<Kmax-Kmin+1; d++)
	{
		while (i = dijkstraArrayFirst[d])
		{
			assert(i->xMax - i->x == d);

			dijkstraArrayFirst[d] = i->next;
			if (dijkstraArrayFirst[d]) dijkstraArrayFirst[d]->prev = NULL;

			for (e=i->firstForward; e; e=e->nextForward)
			{
				j = e->j;
				int diff = e->V.GetMaxNonincreasing(j->x - i->x, j->xMax - i->xMax);
				if (diff < j->xMax - i->xMax)
				{
					if (j->prev) j->prev->next = j->next;
					else         dijkstraArrayFirst[j->xMax - j->x] = j->next;
					if (j->next) j->next->prev = j->prev;

					j->xMax = i->xMax + diff;

					j->next = dijkstraArrayFirst[j->xMax - j->x];
					if (dijkstraArrayFirst[j->xMax - j->x]) dijkstraArrayFirst[j->xMax - j->x]->prev = j;
					dijkstraArrayFirst[j->xMax - j->x] = j;
					j->prev = NULL;
				}
			}

			j = i;
			for (e=j->firstBackward; e; e=e->nextBackward)
			{
				i = e->i;
				int diff = e->V.GetMinNondecreasing(j->x - i->x, j->xMax - i->xMax);
				if (diff > j->xMax - i->xMax)
				{
					if (i->prev) i->prev->next = i->next;
					else         dijkstraArrayFirst[i->xMax - i->x] = i->next;
					if (i->next) i->next->prev = i->prev;

					i->xMax = j->xMax - diff;

					i->next = dijkstraArrayFirst[i->xMax - i->x];
					if (dijkstraArrayFirst[i->xMax - i->x]) dijkstraArrayFirst[i->xMax - i->x]->prev = i;
					dijkstraArrayFirst[i->xMax - i->x] = i;
					i->prev = NULL;
				}
			}

			if (j->xMax != j->x)
			{
				j->D.Increase(j->xMax);
				isChanged = true;
			}
		}
	}

	if (!isChanged) return false;

	for (i=nodes; i<nodes+nodeNum; i++)
	{
		i->x = i->xMax;
		for (e=i->firstForward; e; e=e->nextForward)
		{
			e->V.Init(e->j->xMax - i->xMax);
		}
	}

	return true;
}

bool ConvexMRF::Down()
{
	GraphType::node_id _i;
	Node* i;
	Node* j;
	Edge* e;
	REAL gradUp, gradDown, flow;
	Label d;
	bool isChanged = false;

	/////////////////////////
	//     MaxflowDown     //
	/////////////////////////
	graph->add_node(nodeNum);
	for (_i=0, i=nodes; i<nodes+nodeNum; _i++, i++)
	{

		gradDown = (i->x == Kmin) ? -REALMAX : i->D.GetGradientDown(i->x);
		graph->add_tweights(_i, gradDown, 0);

		for (e=i->firstForward; e; e=e->nextForward)
		{
			j = e->j;
			if (!e->V.isBreakpoint(j->x - i->x, gradUp, gradDown)) continue;

			graph->add_edge(_i, (int)(j - nodes), gradUp, -gradDown);
		}
	}

	graph->maxflow();

	GraphType::arc_id a = graph->get_first_arc();
	for (i=nodes; i<nodes+nodeNum; i++)
	{
		for (e=i->firstForward; e; e=e->nextForward)
		{
			j = e->j;
			if (!e->V.isBreakpoint(j->x - i->x, gradUp, gradDown)) continue;

			flow = graph->get_rcap(a) - gradUp;

			i->D.flow += flow;
			j->D.flow -= flow;
			e->V.flow += flow;

			a = graph->get_next_arc(a);
			a = graph->get_next_arc(a);
		}

		// initialize DijsktraDown
		d = i->x - i->D.GetMinNondecreasing(i->x, Kmin);
		i->next = dijkstraArrayFirst[d];
		if (dijkstraArrayFirst[d]) dijkstraArrayFirst[d]->prev = i;
		dijkstraArrayFirst[d] = i;
		i->prev = NULL;
		i->xMin = i->x - d;
	}

	graph->reset();

	/////////////////////////
	//     DijkstraDown    //
	/////////////////////////
	for (d=0; d<Kmax-Kmin+1; d++)
	{
		while (i = dijkstraArrayFirst[d])
		{
			assert(i->x - i->xMin == d);

			dijkstraArrayFirst[d] = i->next;
			if (dijkstraArrayFirst[d]) dijkstraArrayFirst[d]->prev = NULL;

			for (e=i->firstForward; e; e=e->nextForward)
			{
				j = e->j;
				int diff = e->V.GetMinNondecreasing(j->x - i->x, j->xMin - i->xMin);
				if (diff > j->xMin - i->xMin)
				{
					if (j->prev) j->prev->next = j->next;
					else         dijkstraArrayFirst[j->x - j->xMin] = j->next;
					if (j->next) j->next->prev = j->prev;

					j->xMin = i->xMin + diff;

					j->next = dijkstraArrayFirst[j->x - j->xMin];
					if (dijkstraArrayFirst[j->x - j->xMin]) dijkstraArrayFirst[j->x - j->xMin]->prev = j;
					dijkstraArrayFirst[j->x - j->xMin] = j;
					j->prev = NULL;
				}
			}

			j = i;
			for (e=j->firstBackward; e; e=e->nextBackward)
			{
				i = e->i;
				int diff = e->V.GetMaxNonincreasing(j->x - i->x, j->xMin - i->xMin);
				if (diff < j->xMin - i->xMin)
				{
					if (i->prev) i->prev->next = i->next;
					else         dijkstraArrayFirst[i->x - i->xMin] = i->next;
					if (i->next) i->next->prev = i->prev;

					i->xMin = j->xMin - diff;

					i->next = dijkstraArrayFirst[i->x - i->xMin];
					if (dijkstraArrayFirst[i->x - i->xMin]) dijkstraArrayFirst[i->x - i->xMin]->prev = i;
					dijkstraArrayFirst[i->x - i->xMin] = i;
					i->prev = NULL;
				}
			}

			if (j->xMin != j->x)
			{
				j->D.Decrease(j->xMin);
				isChanged = true;
			}
		}
	}

	if (!isChanged) return false;

	for (i=nodes; i<nodes+nodeNum; i++)
	{
		i->x = i->xMin;
		for (e=i->firstForward; e; e=e->nextForward)
		{
			e->V.Init(e->j->xMin - i->xMin);
		}
	}

	return true;
}

bool ConvexMRF::DijkstraUp()
{
	Node* i;
	Node* j;
	Edge* e;
	Label d;
	bool isChanged = false;

	for (i=nodes; i<nodes+nodeNum; i++)
	{
		d = i->D.GetMaxNonincreasing(i->x, Kmax) - i->x;
		i->next = dijkstraArrayFirst[d];
		if (dijkstraArrayFirst[d]) dijkstraArrayFirst[d]->prev = i;
		dijkstraArrayFirst[d] = i;
		i->prev = NULL;
		i->xMax = i->x + d;
	}

	for (d=0; d<Kmax-Kmin+1; d++)
	{
		while (i = dijkstraArrayFirst[d])
		{
			assert(i->xMax - i->x == d);

			dijkstraArrayFirst[d] = i->next;
			if (dijkstraArrayFirst[d]) dijkstraArrayFirst[d]->prev = NULL;

			for (e=i->firstForward; e; e=e->nextForward)
			{
				j = e->j;
				int diff = e->V.GetMaxNonincreasing(j->x - i->x, j->xMax - i->xMax);
				if (diff < j->xMax - i->xMax)
				{
					if (j->prev) j->prev->next = j->next;
					else         dijkstraArrayFirst[j->xMax - j->x] = j->next;
					if (j->next) j->next->prev = j->prev;

					j->xMax = i->xMax + diff;

					j->next = dijkstraArrayFirst[j->xMax - j->x];
					if (dijkstraArrayFirst[j->xMax - j->x]) dijkstraArrayFirst[j->xMax - j->x]->prev = j;
					dijkstraArrayFirst[j->xMax - j->x] = j;
					j->prev = NULL;
				}
			}

			j = i;
			for (e=j->firstBackward; e; e=e->nextBackward)
			{
				i = e->i;
				int diff = e->V.GetMinNondecreasing(j->x - i->x, j->xMax - i->xMax);
				if (diff > j->xMax - i->xMax)
				{
					if (i->prev) i->prev->next = i->next;
					else         dijkstraArrayFirst[i->xMax - i->x] = i->next;
					if (i->next) i->next->prev = i->prev;

					i->xMax = j->xMax - diff;

					i->next = dijkstraArrayFirst[i->xMax - i->x];
					if (dijkstraArrayFirst[i->xMax - i->x]) dijkstraArrayFirst[i->xMax - i->x]->prev = i;
					dijkstraArrayFirst[i->xMax - i->x] = i;
					i->prev = NULL;
				}
			}

			if (j->xMax != j->x)
			{
				j->D.Increase(j->xMax);
				isChanged = true;
			}
		}
	}

	if (!isChanged) return false;

	for (i=nodes; i<nodes+nodeNum; i++)
	{
		i->x = i->xMax;
		for (e=i->firstForward; e; e=e->nextForward)
		{
			e->V.Init(e->j->xMax - i->xMax);
		}
	}

	return true;
}

bool ConvexMRF::DijkstraDown()
{
	Node* i;
	Node* j;
	Edge* e;
	Label d;
	bool isChanged = false;

	for (i=nodes; i<nodes+nodeNum; i++)
	{
		d = i->x - i->D.GetMinNondecreasing(i->x, Kmin);
		i->next = dijkstraArrayFirst[d];
		if (dijkstraArrayFirst[d]) dijkstraArrayFirst[d]->prev = i;
		dijkstraArrayFirst[d] = i;
		i->prev = NULL;
		i->xMin = i->x - d;
	}

	for (d=0; d<Kmax-Kmin+1; d++)
	{
		while (i = dijkstraArrayFirst[d])
		{
			assert(i->x - i->xMin == d);

			dijkstraArrayFirst[d] = i->next;
			if (dijkstraArrayFirst[d]) dijkstraArrayFirst[d]->prev = NULL;

			for (e=i->firstForward; e; e=e->nextForward)
			{
				j = e->j;
				int diff = e->V.GetMinNondecreasing(j->x - i->x, j->xMin - i->xMin);
				if (diff > j->xMin - i->xMin)
				{
					if (j->prev) j->prev->next = j->next;
					else         dijkstraArrayFirst[j->x - j->xMin] = j->next;
					if (j->next) j->next->prev = j->prev;

					j->xMin = i->xMin + diff;

					j->next = dijkstraArrayFirst[j->x - j->xMin];
					if (dijkstraArrayFirst[j->x - j->xMin]) dijkstraArrayFirst[j->x - j->xMin]->prev = j;
					dijkstraArrayFirst[j->x - j->xMin] = j;
					j->prev = NULL;
				}
			}

			j = i;
			for (e=j->firstBackward; e; e=e->nextBackward)
			{
				i = e->i;
				int diff = e->V.GetMaxNonincreasing(j->x - i->x, j->xMin - i->xMin);
				if (diff < j->xMin - i->xMin)
				{
					if (i->prev) i->prev->next = i->next;
					else         dijkstraArrayFirst[i->x - i->xMin] = i->next;
					if (i->next) i->next->prev = i->prev;

					i->xMin = j->xMin - diff;

					i->next = dijkstraArrayFirst[i->x - i->xMin];
					if (dijkstraArrayFirst[i->x - i->xMin]) dijkstraArrayFirst[i->x - i->xMin]->prev = i;
					dijkstraArrayFirst[i->x - i->xMin] = i;
					i->prev = NULL;
				}
			}

			if (j->xMin != j->x)
			{
				j->D.Decrease(j->xMin);
				isChanged = true;
			}
		}
	}

	if (!isChanged) return false;

	for (i=nodes; i<nodes+nodeNum; i++)
	{
		i->x = i->xMin;
		for (e=i->firstForward; e; e=e->nextForward)
		{
			e->V.Init(e->j->xMin - i->xMin);
		}
	}

	return true;
}

bool ConvexMRF::TestGlobalMin(int _Kmin, int _Kmax)
{
	Node* i;
	Node* j;
	Edge* e;
	bool isGlobalMin = true;

	for (j=nodes; j<nodes+nodeNum; j++)
	{
		j->D.Init(j->x);
		j->D.flow = 0;

		for (e=j->firstBackward; e; e=e->nextBackward)
		{
			i = e->i;
			e->V.Init(j->x - i->x);
			i->D.flow += e->V.flow;
			j->D.flow -= e->V.flow;
		}
	}

	for (i=nodes; i<nodes+nodeNum; i++)
	{
		assert(i->x>=_Kmin && i->x<=_Kmax);
		if (i->x < _Kmax && i->D.GetGradientUp(i->x) < 0) isGlobalMin = false;
		if (i->x > _Kmin && i->D.GetGradientDown(i->x) > 0) isGlobalMin = false;

		for (e=i->firstForward; e; e=e->nextForward)
		{
			j = e->j;
			if (e->V.GetGradientUp(j->x - i->x) < 0) isGlobalMin = false;
			if (e->V.GetGradientDown(j->x - i->x) > 0) isGlobalMin = false;
		}
	}

	return isGlobalMin;
}

void ConvexMRF::ProcessPriorities(int* priorities)
{
	Node* i;
	Node* j;
	Edge* e;
	Edge* e2;
	int p, p_min = priorities[0], p_max = priorities[0];
	int node_num_tmp, edge_num_tmp;
	int* node_num_array;
	int* edge_num_array;
	int* p_ptr;
	int num;

	// computing min and max priority values, number of nodes for each value
	for (p_ptr=priorities; p_ptr<priorities+nodeNum; p_ptr++)
	{
		if (p_min > *p_ptr) p_min = *p_ptr;
		if (p_max < *p_ptr) p_max = *p_ptr;
	}
	if (p_min == p_max) return;
	num = p_max - p_min + 1;
	node_num_array = new int[num];
	edge_num_array = new int[num];
	for (p=0; p<p_max-p_min+1; p++) 
	{
		node_num_array[p] = edge_num_array[p] = 0;
	}
	for (j=nodes, p_ptr=priorities; j<nodes+nodeNum; j++, p_ptr++)
	{
		j->id2 = p_min - *p_ptr - 1; // \in [-num,-1]
		node_num_array[-j->id2-1] ++;
		for (e=j->firstBackward; e; e=e->nextBackward)
		{
			p = (e->i->id2 < j->id2) ? e->i->id2 : j->id2;
			edge_num_array[-p-1] ++;
		}
	}

	// main loop
	for (p=-1, node_num_tmp=node_num_array[0], edge_num_tmp=edge_num_array[0]; 
		p>-num; 
		p--, node_num_tmp+=node_num_array[-p-1], edge_num_tmp+=edge_num_array[-p-1]
		)
	{
		ConvexMRF* g = new ConvexMRF(node_num_tmp, edge_num_tmp);

		node_num_tmp = 0;
		e2 = g->edges;
		for (j=nodes; j<nodes+nodeNum; j++)
		{
			if (j->id2 < p)
			{
				continue;
			}
			j->id2 = node_num_tmp ++;
			g->nodes[j->id2].D.Copy(j->D);

			g->SetSolution(j->id2, j->x);

			for (e=j->firstBackward; e; e=e->nextBackward)
			{
				i = e->i;
				if (i->id2 < 0) continue;

				e2->i = & g->nodes[i->id2];
				e2->j = & g->nodes[j->id2];
				e2->nextForward = g->nodes[i->id2].firstForward;
				g->nodes[i->id2].firstForward = e2;
				e2->nextBackward = g->nodes[j->id2].firstBackward;
				g->nodes[j->id2].firstBackward = e2;

				e2->V.Copy(e->V);
				e2 ++;
			}
		}

		g->Solve(Kmin, Kmax);

		e2 = g->edges;
		for (j=nodes; j<nodes+nodeNum; j++)
		{
			if (j->id2 < 0) continue;

			j->x = (g->GetSolution(j->id2, 0) + g->GetSolution(j->id2, 1)) / 2;
			for (e=j->firstBackward; e; e=e->nextBackward)
			{
				i = e->i;
				if (i->id2 < 0) continue;

				e->V.flow = e2->V.flow;

				e2 ++;
			}
		}

		delete g;
	}

	delete [] node_num_array;
	delete [] edge_num_array;
}

void ConvexMRF::Solve(Label _Kmin, Label _Kmax, int *priorities)
{
	Node* i;
	Node* j;
	Edge* e;
	bool isChanged;

	Kmin = _Kmin;
	Kmax = _Kmax;

	if (priorities)
	{
		ProcessPriorities(priorities);
	}

	graph = new GraphType(nodeNum, edgeNum);
	dijkstraArrayFirst = new Node*[Kmax-Kmin+1];
	memset(dijkstraArrayFirst, 0, (Kmax-Kmin+1)*sizeof(Node*));

	for (j=nodes; j<nodes+nodeNum; j++)
	{
		if (j->x < Kmin) j->x = Kmin;
		if (j->x > Kmax) j->x = Kmax;
		j->D.Init(j->x);
		j->D.flow = 0;

		for (e=j->firstBackward; e; e=e->nextBackward)
		{
			i = e->i;
			e->V.Init(j->x - i->x);
			e->V.InitFlow(j->x - i->x);
			i->D.flow += e->V.flow;
			j->D.flow -= e->V.flow;
		}
	}

	printf("UP:   ");
	do
	{
		isChanged = Up();
		printf("*");
	} while (isChanged);
	printf("\n");

	printf("DOWN: ");
	do
	{
		isChanged = Down();
		printf("*");
	} while (isChanged);
	printf("\n");

	DijkstraUp();

//	if (TestGlobalMin(Kmin, Kmax)) printf("success\n");
//	else                           { printf("failure\n"); exit(1); }

	delete [] dijkstraArrayFirst;
	delete graph;
}

