/* graph.h */
/*
	This is a software for minimizing ``convex MRF energy functions'', i.e. functions
	of the form

	E(x1, ..., xn) = \sum_{i} Di(xi) + \sum_{(i,j)} Vij(xj-xi)

	where xi \in [Kmin,Kmax] are integer variables, 
	and Di() and Vij() are convex functions. This library implements 
	the primal-dual algorithm described in

		"New Algorithms for the Dual of the Convex Cost Network Flow Problem with Application to Computer Vision"
		Vladimir Kolmogorov and Akiyoshi Shioura
		Technical Report, June 2007.

	It uses maxflow algorithm described in

		"An Experimental Comparison of Min-Cut/Max-Flow Algorithms for Energy Minimization in Vision"
		Yuri Boykov and Vladimir Kolmogorov.
		In IEEE Transactions on Pattern Analysis and Machine Intelligence (PAMI), 
		September 2004.

	If you use this software for research purposes, you should cite
	the aforementioned papers in any resulting publication.
*/
	


/*
	For description, license, example usage see README.TXT.
*/

#ifndef __CONFEX_MRF__
#define __CONFEX_MRF__

#include "maxflow/graph.h"

#include "assert.h"


class ConvexMRF
{
public:
	typedef int NodeId; // integer in {0,1,...,nodeNum-1}
	typedef int Label;
	typedef int REAL; 
	typedef Graph<int,int,int> GraphType;
	const static REAL REALMAX = (REAL)((unsigned)(-1)/4);

	class ConvexTerm;


	// Constructor. 
	// The last (optional) argument is the pointer to the function which will be called 
	// if an error occurs; an error message is passed to this function. 
	// If this argument is omitted, exit(1) will be called.
	ConvexMRF(int nodeNum, int edgeNumMax, void (*err_function)(char *) = NULL);

	// Destructor.
	~ConvexMRF();

	// Adds unary term Di(xi) to the energy. Can be called at most once per node.
	void AddUnaryTerm(NodeId i, ConvexTerm& Di);

	// Adds a pairwise term Vij(xj-xi) to the energy. May be called at most edgeNumMax times.
	void AddPairwiseTerm(NodeId i, NodeId j, ConvexTerm& Vij);

	// Primal-dual algorithm.
	// If optional argument 'priorities' is given, then
	// the problem is first solved for the subgraph specified by 'priorities'.
	// (For the panoramic image stitching problem this is much faster.)
	// 'priorities' is an array of size nodeNum containing integers 0,1,...
	// First nodes with priorities 0 are considered, then
	// the nodes with priorities 0 and 1, then 0,1 and 2, and so on.
	void Solve(Label Kmin, Label Kmax, int* priorities = NULL);

	void SetSolution(NodeId i, Label x);

	// if solution_option = 0 then the minimal solution is returned, otherwise maximal solution.
	Label GetSolution(NodeId i, int solution_option = 0);


	class ConvexTerm
	{
	public:
		// Constructor. 
		// The last (optional) argument is the pointer to the function which will be called 
		// if an error occurs; an error message is passed to this function. 
		// If this argument is omitted, exit(1) will be called.
		ConvexTerm(int breakpointNumMax, void (*err_function)(char *) = NULL);

		// Destructor.
		~ConvexTerm();

		// Adds the following term f(x):
		//  f(x) = s1*x   if x<breakpoint
		//         s2*x   if x>breakpoint
		// f must be convex, i.e. there must hold s1<=s2.
		//
		// May be called at most breakpointNumMax times.
		void AddLinearTerm(Label breakpoint, REAL s1, REAL s2);

		// Removes all terms added so far
		void Reset();

	protected:
		friend class ConvexMRF;
		int		breakpointNum, breakpointNumMax;
		REAL	slope0;
		struct Point 
		{
			Label breakpoint; 
			REAL slope; // slope for labels x>=breakpoint
		}*		 points; // array of size breakpointNumMax;
		Point**	buf; // for mergesort
	};
















/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////

private:

	struct ConvexTermInternal;
	struct Node;
	struct Edge;

	struct ConvexTermInternal
	{
		int		breakpointNum;
		Label*	breakpoints; // points to array of size breakpointNum
		REAL*	slopes; // points to array of size breakpointNum+1

		REAL	flow;

		void Init(Label xNew);
		void InitFlow(Label xCurrent);
		void Increase(Label xNew);
		void Decrease(Label xNew);
		Label GetMaxNonincreasing(Label xCurrent, Label Kmax);
		Label GetMinNondecreasing(Label xCurrent, Label Kmin);
		REAL GetGradientUp(Label xCurrent);
		REAL GetGradientDown(Label xCurrent);
		bool isBreakpoint(Label xCurrent, REAL& gradUp, REAL& gradDown);

		void Copy(ConvexTermInternal& from);
	private:
		int	b; // index s.t. breakpoints[b-1] <= x < breakpoints[b] where x is the current label
		       // 0 <= b <= breakpointNum (convention: breakpoints[-1] = -infty, breakpoints[breakpointNum] = + infty)
	};

	struct Node
	{
		ConvexTermInternal	D;
		Label	x, xMax, xMin;

		Edge*	firstForward;
		Edge*	firstBackward;

		union
		{
			// for Dijkstra
			struct
			{
				Node*	prev;
				Node*	next;
			};
			union
			{
				GraphType::node_id id;
				NodeId id2;
			};
		};
	};

	struct Edge
	{
		ConvexTermInternal	V;

		Node*	i;
		Node*	j;
		Edge*	nextForward;
		Edge*	nextBackward;
	};

	Node*	nodes;
	Edge*	edges;

	int		nodeNum;
	int		edgeNum, edgeNumMax;

	void	(*error_function)(char *);	// this function is called if a error occurs,
										// with a corresponding error message
										// (or exit(1) is called if it's NULL)


	void ConvertConvexTerm(ConvexTermInternal& to, ConvexTerm& from, bool reverse = false);

	// temporary variables - valid only during Solve()
	Label	Kmin, Kmax;
	Node**	dijkstraArrayFirst; //  array of size Kmax-Kmin+1
	GraphType*	graph;

	void ProcessPriorities(int* priorities);

	// Up() and Down() return true if the labeling has changed.
	bool Up(); // = MaxflowUP + DijkstraUP
	bool Down(); // = MaxflowDOWN + DijkstraDOWN

	bool DijkstraUp();
	bool DijkstraDown();

	bool TestGlobalMin(int Kmin, int Kmax); // debug function

	// Memory management
	void* Malloc(int sizeInBytes, void (*err_function)(char *) = NULL);

	struct MemoryBlock
	{
		static const int SIZE_MIN = 1024;
		MemoryBlock*	next;
		char*			firstFree;
		char			data[1]; // actual size of data is 'size'.
	};

	MemoryBlock*	memoryBlockFirst;
	MemoryBlock*	memoryBlockLast;
};

inline void ConvexMRF::ConvexTerm::AddLinearTerm(Label breakpoint, REAL s1, REAL s2)
{
	assert(breakpointNum < breakpointNumMax);
	assert(s1 <= s2);

	slope0 += s1;
	if (s2 > s1)
	{
		points[breakpointNum].breakpoint = breakpoint;
		points[breakpointNum].slope = s2 - s1;
		breakpointNum ++;
	}
}

inline void ConvexMRF::AddUnaryTerm(NodeId i, ConvexTerm& Di)
{
	assert(i>=0 && i<nodeNum);

	ConvertConvexTerm(nodes[i].D, Di);
}

inline void ConvexMRF::AddPairwiseTerm(NodeId _i, NodeId _j, ConvexTerm& Vij)
{
	assert(edgeNum < edgeNumMax);

	Node* i = nodes + _i;
	Node* j = nodes + _j;
	Edge* e = & edges[edgeNum ++];

	if (_i < _j)
	{
		e->i = i;
		e->j = j;
		e->nextForward = i->firstForward;
		i->firstForward = e;
		e->nextBackward = j->firstBackward;
		j->firstBackward = e;

		ConvertConvexTerm(e->V, Vij, false);
	}
	else
	{
		e->i = j;
		e->j = i;
		e->nextForward = j->firstForward;
		j->firstForward = e;
		e->nextBackward = i->firstBackward;
		i->firstBackward = e;

		ConvertConvexTerm(e->V, Vij, true);
	}
}

inline void ConvexMRF::SetSolution(NodeId i, Label x)
{
	assert(i>=0 && i<nodeNum);
	nodes[i].x = x;
}

inline ConvexMRF::Label ConvexMRF::GetSolution(NodeId i, int solution_option)
{
	assert(i>=0 && i<nodeNum);
	return (solution_option == 0) ? nodes[i].xMin : nodes[i].xMax;
}

inline void ConvexMRF::ConvexTerm::Reset()
{
	breakpointNum = 0;
	slope0 = 0;
}

inline void ConvexMRF::ConvexTermInternal::Init(Label xNew)
{
	for (b=0; b<breakpointNum; b++)
	{
		if (breakpoints[b] > xNew) break;
	}

	assert(b>=0 && b<=breakpointNum);
	assert(b==0 || breakpoints[b-1]<=xNew);
	assert(b==breakpointNum || breakpoints[b]>xNew);
}

inline void ConvexMRF::ConvexTermInternal::InitFlow(Label xCurrent)
{
	assert(b>=0 && b<=breakpointNum);
	assert(b==0 || breakpoints[b-1]<=xCurrent);
	assert(b==breakpointNum || breakpoints[b]>xCurrent);

	int flow_old = flow;

	if (b>0 && breakpoints[b-1] == xCurrent)
	{
		if (flow > -slopes[b-1]) flow = -slopes[b-1];
		if (flow < -slopes[b])   flow = -slopes[b];
	}
	else flow = -slopes[b];
}

inline void ConvexMRF::ConvexTermInternal::Increase(Label xNew)
{
	assert(b>=0 && b<=breakpointNum);
	assert(b==0 || breakpoints[b-1]<=xNew);

	for ( ; b<breakpointNum; b++)
	{
		if (breakpoints[b] > xNew) break;
	}

	assert(b>=0 && b<=breakpointNum);
	assert(b==0 || breakpoints[b-1]<=xNew);
	assert(b==breakpointNum || breakpoints[b]>xNew);
}

inline void ConvexMRF::ConvexTermInternal::Decrease(Label xNew)
{
	assert(b>=0 && b<=breakpointNum);
	assert(b==breakpointNum || breakpoints[b]>xNew);

	for ( ; b>0; b--)
	{
		if (breakpoints[b-1] <= xNew) break;
	}

	assert(b>=0 && b<=breakpointNum);
	assert(b==0 || breakpoints[b-1]<=xNew);
	assert(b==breakpointNum || breakpoints[b]>xNew);
}

inline ConvexMRF::Label ConvexMRF::ConvexTermInternal::GetMaxNonincreasing(Label xCurrent, Label Kmax)
{
	assert(b>=0 && b<=breakpointNum);
	assert(b==0 || breakpoints[b-1]<=xCurrent);
	assert(b==breakpointNum || breakpoints[b]>xCurrent);

	Label bb;

	if (slopes[b] > -flow) return xCurrent;

	for (bb=b; bb<breakpointNum; bb++)
	{
		if (slopes[bb+1] > -flow)
		{
			return (breakpoints[bb] < Kmax) ? breakpoints[bb] : Kmax;
		}
	}

	return Kmax;
}

inline ConvexMRF::Label ConvexMRF::ConvexTermInternal::GetMinNondecreasing(Label xCurrent, Label Kmin)
{
	assert(b>=0 && b<=breakpointNum);
	assert(b==0 || breakpoints[b-1]<=xCurrent);
	assert(b==breakpointNum || breakpoints[b]>xCurrent);

	Label bb;

	if (slopes[b] < -flow) return xCurrent;

	for (bb=b-1; bb>=0; bb--)
	{
		if (slopes[bb] < -flow)
		{
			return (breakpoints[bb] > Kmin) ? breakpoints[bb] : Kmin;
		}
	}

	return Kmin;
}

inline ConvexMRF::REAL ConvexMRF::ConvexTermInternal::GetGradientUp(Label xCurrent)
{
	assert(b>=0 && b<=breakpointNum);
	assert(b==0 || breakpoints[b-1]<=xCurrent);
	assert(b==breakpointNum || breakpoints[b]>xCurrent);

	return slopes[b] + flow;
}

inline ConvexMRF::REAL ConvexMRF::ConvexTermInternal::GetGradientDown(Label xCurrent)
{
	assert(b>=0 && b<=breakpointNum);
	assert(b==0 || breakpoints[b-1]<=xCurrent);
	assert(b==breakpointNum || breakpoints[b]>xCurrent);

	if (b!=0 && breakpoints[b-1] == xCurrent) return slopes[b-1] + flow;
	else                                      return slopes[b] + flow;
}

inline bool ConvexMRF::ConvexTermInternal::isBreakpoint(Label xCurrent, REAL& gradUp, REAL& gradDown)
{
	assert(b>=0 && b<=breakpointNum);
	assert(b==0 || breakpoints[b-1]<=xCurrent);
	assert(b==breakpointNum || breakpoints[b]>xCurrent);

	if (b!=0 && breakpoints[b-1] == xCurrent)
	{
		gradDown = slopes[b-1] + flow;
		gradUp   = slopes[b] + flow;
		return true;
	}
	else return false;
}

inline void ConvexMRF::ConvexTermInternal::Copy(ConvexTermInternal& from)
{
	breakpointNum = from.breakpointNum;
	breakpoints = from.breakpoints;
	slopes = from.slopes;
	flow = from.flow;
}

inline void* ConvexMRF::Malloc(int sizeInBytes, void (*err_function)(char *))
{
	if (!memoryBlockLast || memoryBlockLast->firstFree + sizeInBytes > &memoryBlockLast->data[MemoryBlock::SIZE_MIN])
	{
		int size = (sizeInBytes > MemoryBlock::SIZE_MIN) ? sizeInBytes : MemoryBlock::SIZE_MIN;
		MemoryBlock* b = (MemoryBlock*) new char[sizeof(MemoryBlock) - sizeof(char) + size];
		if (!b) { if (err_function) (*err_function)("Not enough memory!"); exit(1); }
		b -> firstFree = & b->data[0];
		b -> next = NULL;
		if (memoryBlockLast) memoryBlockLast->next = b;
		else                 memoryBlockFirst = b;
		memoryBlockLast = b;
	}

	void* ptr = memoryBlockLast -> firstFree;
	memoryBlockLast -> firstFree += sizeInBytes;
	return ptr;
}

#endif
